@@ -7,7 +7,7 @@ test_that("image objects from oro.nifti can be read", {
     
     # The oro.nifti package warns about nonzero slope, which is nothing to worry about
     image <- suppressWarnings(oro.nifti::readNIfTI(imagePath))
-    expect_equal(dumpNifti(image)$bitpix, 32L)
+    expect_equal(niftiHeader(image)$bitpix, 32L)
 })
 
 test_that("image objects from tractor.base can be read", {
@@ -17,9 +17,10 @@ test_that("image objects from tractor.base can be read", {
     
     reportr::setOutputLevel(reportr::OL$Warning)
     
+    # NB: the $ operator shortcut can't be used since "image" isn't a niftiImage
     image <- tractor.base::readImageFile(imagePath)
-    expect_equal(dumpNifti(image)$bitpix, 32L)
+    expect_equal(niftiHeader(image)$bitpix, 32L)
     
     image <- tractor.base::readImageFile(imagePath, sparse=TRUE)
-    expect_equal(dumpNifti(image)$bitpix, 32L)
+    expect_equal(niftiHeader(image)$bitpix, 32L)
 })
